#include <iostream>		//Control de flujo de entrada y salida
#include <string> 		//Trabajar con cadenas de texto
#include <vector>		//Arrays de almacenamiento dinamico
#include <map>			//Contenedor de los hash	
#include <algorithm>	//Algoritmos utiles (find, abs...)
#include <fstream>		//Control del stream de I/O por archivos
#include <sstream>		//Control del stream de strings
#include <ctime>		//Trabajar con el tiempo

using namespace std;

typedef char tablero [71]; /**64 casillas + 4 que idican los derechos de enroque
					  * + 1 que indica el turno + 1 que indica las capturas al paso + 1 que indica la ultima captura**/


/* Derechos de eroque (0: no se puede, 1: se puede)
 * la posicion 64 indica si el blanco se puede enrocar por el flanco rey.
 * la 65 indica si el blanco se puede enrocar por el flanco dama.
 * la 66 indica si el negro puede por el flanco rey
 * la 67 si el negro por el flanco dama 
 *
 * turno: posicion 68-->1 blancas, 0 negras (es como si fuera un bool que tiene "le toca al blanco") 
 * 
 * 69-->La captura a paso indica la casilla que se puede capturar.
 * 
 * Posicion 70 ultima captura
 */
/* A definir:
 * valores positivos--> blancas, negativo--> negras, 0=vacio
 * 1: peon
 * 2: caballo
 * 3: alfil
 * 4: torre
 * 5: dama
 * 6: rey
 */

/***********************************
 *        Variables Golabales      *
 *                                 * 
 ***********************************/ 

tablero principal;
bool maquina_es_blancas = false, maquina_es_negras = false;
unsigned long long int nodos;
vector  <int> mejormove (10, 0);
int prof = 4, qprof = 2; //con esto es 1min/jugada +-, aunque no se usa la qsearch.

unsigned long long int zobrist [12][64];	//zobrist[pieza][cuadro] 
map<unsigned long long int, int> valoraciones;
map<unsigned long long int, int> ultima_mejorjugada;
map<unsigned long long int, int> evaluaciones;
map<unsigned long long int, short int> posiciones;
unsigned long long int position_key;
bool posicion_anterior;

/***********************************
 *     Todas las funciones con     *
 *      una breve descripcion      * 
 ***********************************/ 

/**Makemove hace el movimiento**/
void makemove(int move, tablero &main, bool actualiza_key);
/** Deshace el movimiento "move" en el tablero "main"
 *  La array_final contiene los datos que no son cuadros (derechos de enroque...) de la posicion resultante
 *  El septimo elemento dice lo que habia en la casilla from (por si captura, o si corona)
 * **/
void unmakemove(int move, tablero &main, const vector <int> array_final, bool actualiza_key);
/**Muestra el tablero por pantalla**/
void print_board(const tablero &input);
/**Esta funcion genera todos los posibles movimientos de la pieza en la posicion "from"
 * se preocupara si es legal (pone al rey en jaque) si siguiente es true.
 * **/
vector<int> gen_moves(tablero &estado, const int &from, const bool &siguente);
/**Genera tan solo los moves de jaque o que matan**/
vector<int> gen_caps(tablero &estado, const int &from, const bool &siguente);
/**Untilizando las dos funciones anteriores genera todos los movimientos de "blancas" con "capturas o capturas y todo lo demas**/
vector<int> gen_all_moves(tablero &estado, bool blancas, bool capturas);
/**Una funcion que retorna true si una casilla esta siendo atacada por el bando (1 blancas -1 negras) en la posicion actual. */
bool ataca(const bool &blancas,const int &casilla, tablero &principal);
/**Pone el tablero en la posicion inicial**/
void inicialize_board(tablero &input);
/**Transforma un entero que nadie entiene en un bonito string**/	
string enterohumano(int input, tablero &board, bool ingles);
/**Determina si el movimiento "move" es legal en el tablero "board" y retorna true si lo es**/
bool correct_move(tablero &board, int move);
/**Determina si la posicion del tablero "board" el bando "bando" no tiene movimientos**/
bool no_moves(tablero &board,const bool &blancas);
/**Si la funcion anterior es cierta, se llama a esta para asegurar que esta en mate, no en ahogado**/
bool ahogado(tablero &board, bool blancas);
/**Simplemente dice si el rey de color "bando" esta en jaque o no**/
bool jaque(tablero &board, bool blancas);
/**Hace el log de la partida, donde los movimientos estan guardados en el vector partida**/
void hacer_log(const vector <int> &partida);
/**Un pequeNo menu a prueva de tontos**/
int menu_a_prueva(tablero &board);
/**Tercer parametro para la funcion "sort"**/
bool ordenar(const int &a, const int &b);
/**Busqueda en la posicion "board" con limite superior "alpha" y limite inferior "beta" a una profundidad**/
int search(tablero &board, int alpha, int beta, int profundidad);
/**Identica que "search", solo que aNade tres niveles de profundidad con gen_caps**/
int qsearch(tablero &board, int alpha, int beta, int profundidad);
/**Calcula los pros y los contras de la estructura de peones de ambos bandos**/
int estructura_de_peones(vector <int> &columnas_blancas, vector <int> &columnas_negras);
/**Devuelve un valor numerico segun la ventaja de "blancas" en la posicion  "board"**/
int evaluate(tablero &board, bool blancas);
/**Calcula la key del hash de la posicion "b"**/
unsigned long long int calcula_key(const tablero &b);
/**Genera numeros al azar del tipo long long**/
unsigned long long int lngrand();
/**Le da valores al azar de zobrist**/
void inicia_hash();

/***********************************
 * 		Funciones de entrada y     *
 * 			salida                 * 
 ***********************************/ 
void print_board(const tablero &input){
	cout<<"  +---+---+---+---+---+---+---+---+"<<endl;
	if(not input[68]) cout<<"1 ";
	else cout<<"8 ";
	for(int j=0; j<64; ++j){
		int i;
		if(not input[68]) i=j;
		else i=63-j;
		string pieza = " PCATDRpcatdr";
		int indice_pieza = input[i];
		cout<<"|";
		if (input[i]>0) cout << "<";
		else if (input[i]<0){
			cout << "-";
			indice_pieza = indice_pieza*-1 + 6;
		}
		else if(input[i] == 0){
			int aux=(int)i/8;
			if((i+aux)%2==0) cout<<"   ";
			else cout << " . ";
			}
		if(input[i] != 0) cout << pieza[indice_pieza];
		if (input[i]>0) cout<<">";
		if (input[i]<0) cout<<"-";
		if(not input[68]){
			if((i+1)%8==0){
				cout<<"|"<<endl;
				cout<<"  +---+---+---+---+---+---+---+---+";
				if(i != 63)cout<<endl<<((i+1)/8)+1<<" ";
				}
			}
		else{
			if(i%8==0){
				cout<<"|"<<endl;
				cout<<"  +---+---+---+---+---+---+---+---+";
				if(i!=0)cout<<endl<<(i/8)<<" ";
				}
			}
		}
		cout<<endl;
		if(input[68]) cout<<"    A   B   C   D   E   F   G   H"<<endl;
		else             cout<<"    H   G   F   E   D   C   B   A"<<endl;
		cout<<endl;
	}
	
string enterohumano(int input, tablero &board, bool ingles){
	int promote = input/10000;
	input = input % 10000;
	int fr=input/100, to=input%100;
	string retornar = "", letras = " PCATDR";
	if (ingles) letras = " PNBRQK";
	if(abs(board[fr])>1) retornar += letras[abs(board[fr])];
	/**Si se trata de un caballo, alfil, torre o dama, podria haber una confusion:
	 * si hay dos piezas del mismo tipo, que pueden ir a la misma casilla, se debe aNadir la casilla de salida**/
	if(abs(board[fr])!=1 and abs(board[fr])!=6){
        bool anadido_num = false, anadido_letr = false;
		for(int i=0; i<64; ++i){
			if(board[i] == board[fr] and i!=fr){
				vector <int> auxi = gen_moves(board, i, true);
				for(int j = 0; j < (int)auxi.size(); ++j){
					if(auxi[j] == to){
						/**Si la columna es diferente**/
						 if(i%8 != fr%8 and not anadido_letr){
                                if(anadido_num){
                                          char c = retornar[retornar.size()-1];
                                          retornar[retornar.size()-1] = (7-(fr%8))+'a';
                                          retornar += c;
                                          }
                                else retornar += (7-(fr%8))+'a';
                                anadido_letr = true;
                                } 
						 /**Si la fila es diferente**/
						 else if(i/8 != fr/8 and not anadido_num){ 
                                retornar += fr/8+'1';
                                anadido_num = true; 
                                }
						}
					}
				}
			}
	}
	/**Si captura**/
	if((board[fr] > 0 and board[to] < 0) or (board[fr] < 0 and board[to] > 0) or to==board[69]){
		if(abs(board[fr])==1) retornar += 7-fr%8 +'a';
		retornar += 'x';
		}
	retornar += 7-to%8 +'a';
	retornar += to/8 + '1';
	if(promote!=0){
		retornar += '=';
		retornar += letras[promote];
		}
	if(to==board[69] and abs(board[fr]) == 1 and not ingles) retornar += " a.p.";
	/**Enroques cortos y largos**/
	if((input==301 and board[3]==6) or (input==5957 and board[59]==-6)) retornar = "O-O";
	if((input==305 and board[3]==6) or (input==5961 and board[59]==-6)) retornar = "O-O-O";
	tablero aux;
	for(int i = 0; i < 71; ++i) aux[i]=board[i];
	makemove(input + promote*10000, aux, false);
	if((jaque(aux, true) and board[fr]<0) or (jaque(aux, false) and board[fr]>0)){
		retornar += '+';
		if((no_moves(aux, true) and board[fr]<0) or (no_moves(aux,false) and board[fr]>0)){
			retornar.erase(retornar.size()-1); //eliminamos el +
			retornar += '#';
		}
	}
	return retornar;
	}
	
int menu_a_prueva(tablero &board){
	cout << "Que movimiento quieres hacer?" << endl;
	vector <int> moves;
	vector <int> caps;
	for(int i=0; i < 64; ++i){
		if((board[i] > 0 and board[68]) or (board[i] < 0 and not board[68])){
			 vector <int> aux = gen_moves(board, i, true);
			 vector <int> capturas = gen_caps(board, i, true);
			 for(int j=0; j< (int) aux.size(); ++j){
				 aux[j] += i*100;
				 moves.push_back(aux[j]);
			}
			for(int j = 0; j < (int) capturas.size(); ++j){
				capturas[j] += i*100;
				caps.push_back(capturas[j]);
				}
		}
	}
	int anterior;
	for(int i=0; i < (int) moves.size(); ++i){
		if(i == 0) anterior = (moves[i]%10000)/100;
		int actual = (moves[i]%10000)/100;
		if(actual!=anterior) cout << endl;
		anterior = actual;
		cout << i+1 << ":" << enterohumano(moves[i], board, false) << "   ";
	}
	cout << endl;
	string input;
	cin >> input;
	bool compr = false;
	for(int i = 0; i < (int)input.size(); ++i) if(not (input[i] >= '0' and  input[i] <= '9')) compr = true;
	unsigned int num;
	if(not compr) num =atoi(input.c_str());
	while(num == 0 or num > moves.size() or compr){
		cout << "El movimiento no esta en la lista, elija otro" <<endl;
		cin >> input;
		compr = false;
		for(int i = 0; i < (int) input.size(); ++i) if((not (input[i] >= '0' and input[i] <= '9'))) compr = true;
		if(not compr) num = atoi(input.c_str());
	}
	return moves[num-1];
}

void hacer_log(const vector <int> &partida){
	fstream archivo;
	archivo.open ("log.pgn", fstream::out | fstream::app); //Stream de salida
	string respuesta;
	char c = '"';
	getline(cin, respuesta); //Salto de linea
	archivo << endl << endl;
	cout << "De que evento se tratava? ";
	getline(cin, respuesta);
	archivo << "[Event " << c << respuesta << c <<"]" << endl;
	cout << "Donde se jugo? ";
	getline(cin, respuesta);
	archivo << "[Site " << c << respuesta << c << "]" << endl;
	cout << "Que dia?(aaaa.mm.dd) ";
	getline(cin, respuesta);
	archivo << "[Date " << c << respuesta << c <<"]" << endl;
	cout << "Que ronda? ";
	getline(cin, respuesta);
	archivo << "[Round " << c << respuesta << c << "]" << endl;
	cout << "Quien llevaba blancas? ";
	getline(cin, respuesta);
	archivo << "[White " << c << respuesta << c << "]" << endl;
	cout << "Quien llevaba negras? ";
	getline(cin, respuesta);
	archivo << "[Black " << c << respuesta << c << "]" << endl;
	cout << "Con que resultado acabo? (0-1, 1-0 o 0.5-0.5) ";
	getline(cin, respuesta);
	archivo << "[Result " << c << respuesta  << c << "]" << endl;
	tablero reproduciendo;
	inicialize_board(reproduciendo);
	for(int i = 0; i < (int) partida.size(); ++i){
		if(i%2 == 0) archivo << (i/2)+1 << ". ";
		archivo << enterohumano(partida[i], reproduciendo, true) << " ";
		if(i%10 == 0 and i != 0) archivo << endl;
		makemove(partida[i], reproduciendo, false);
		}
	archivo << " " << respuesta << endl;
	archivo.close();
	cout << "Ya esta hecho!" << endl;
	}

/***********************************
 * 		MAKEMOVE, UNMAKEMOVE e     *
 * 			inciar tablero         * 
 ***********************************/ 

/* Comencem per definir un moviment: es un numero de 4 xifres que
 * conte la casella d'arribada(to) i de sortida(fr). les dues primeres xifres (milers y centenes)
 * diu des d'on surt i les segones a on va.
 * Utilitzant el modul 100 i la divisio entera es senzill.
 * En el cas que es coroni es pasara un numero de 5 xifres, la primera diu en quina peCa es transforma
 * y les 4 restants es el mateix.
 */
void makemove(int move, tablero &main, bool actualiza_key){
	int promote = 0;
	if(move>10000){
		promote = move / 10000;
		move = move % 10000;
		}
	int fr=move/100, to=move%100;
	if(actualiza_key){
		if(main[fr] > 0){
			position_key ^= zobrist[main[fr]-1][fr];
			if(promote == 0) position_key ^= zobrist[main[fr]-1][to];
			else position_key ^= zobrist[promote-1][to];
			if(main[to] != 0) position_key ^= zobrist[-main[to]+5][to];
		}
		else{
			position_key ^= zobrist[-main[fr]+5][fr];
			if(promote == 0) position_key ^= zobrist[-main[fr]+5][to];
			else position_key ^= zobrist[promote+5][to];
			if(main[to] != 0) position_key ^= zobrist[main[to]-1][to];
			}
		}
	if(abs(main[fr]) == 6){
		if(move==301){  /**Enroque corto, blancas**/
			if(actualiza_key){
				position_key ^= zobrist[3][0];	//Torre blanca en h1
				position_key ^= zobrist[3][2];  //Torre blanca en f1
			}
			main[2] = main[0];
			main[0] = 0;
		}    
		if(move==305){ /**Enroque largo, blancas**/
			if(actualiza_key){
				position_key ^= zobrist[3][7];	//Torre blanca en a1
				position_key ^= zobrist[3][4];  //Torre blanca en d1
			}
			main[4] = main[7];
			main[7] = 0;
		}
		if(move==5957){ /**Enroque corto, negras**/
			if(actualiza_key){
				position_key ^= zobrist[9][56];  //Torre negra en h8
				position_key ^= zobrist[9][58];  //Torre negra en f8
			}
			main[58] = main[56];
			main[56] = 0;
		}
		if(move==5961){ /**Enroque largo, negras**/
			if(actualiza_key){
				position_key ^= zobrist[9][63];  //Torre negra en a8
				position_key ^= zobrist[9][60];  //Torre negra en d8
			}
			main[60] = main[63];
			main[63] = 0;
		}
	}
	/**Si mata o si se mueve un peon se vuelve a empezar la cuenta de 50 movimientos**/
	if(main[to]!=0 or abs(main[fr])==1) main[70] = 0;
	else main[70] += 1;
	/**Si se mueve las piezas que participan en el enroque, el enroque se niega**/
	if(fr == 0 or fr == 3 or (to == 0 and main[fr] < 0)) main[64] = 0;
	if(fr == 7 or fr == 3 or (to == 7 and main[fr] < 0)) main[65] = 0;
	if(fr == 56 or fr == 59 or (to == 56 and main[fr] > 0)) main[66] = 0;
	if(fr == 63 or fr == 59 or (to == 63 and main[fr] > 0)) main[67] = 0;
	/**Solo se matara al paso si se cumplen dos condiciones:
	* 1: se mueve un peon.
	* 2: a la casilla de captura al paso
	**/	
	if(abs(main[fr])==1 and to == main[69]){
		if(main[fr] == 1){
			if(actualiza_key) position_key ^= zobrist[6][to-8];			//Debera ser un peon negro
			main[to-8]=0; /**La casilla de la fila anterior de la llegada**/
		}
		else{
			if(actualiza_key) position_key ^= zobrist[0][to+8];	//Debera ser un peon blanco
			main[to+8] = 0;
		}
	}
	/**Se crean los derechos de capturas al paso**/
	if(abs(main[fr])==1 and (fr/8)+main[fr]*2==to/8) main[69]=fr+main[fr]*8;
	else main[69] = -1;
	if(promote == 0) main[to]=main[fr];
	else main[to] = main[fr] * promote;     /**Nos aprovechamos que main[to] es o 1 o -1, el signo de la promocion**/
	main[fr] = 0;
	main[68]=1-main[68]; //cambi de torn
	if(actualiza_key){
		position_key ^= not main[68];
		position_key ^= main[68];
	}
}

void unmakemove(int move, tablero &main, const vector <int> array_final, bool actualiza_key){
	bool promote = move > 10000;
	for(int i = 0; i < 7; ++i) main[64+i] = array_final[i];
	move = move % 10000;
	int fr = move/100, to = move%100;
	if(actualiza_key){
		if(main[to] > 0){
			position_key ^= zobrist[main[to]-1][to];
			if(promote == 0) position_key ^= zobrist[main[to]-1][fr];
			else position_key ^= zobrist[0][fr];
			if(array_final[7] != 0) position_key ^= zobrist[-array_final[7]+5][to];
			}
		else{
			position_key ^= zobrist[-main[to]+5][to];
			if(promote == 0) position_key ^= zobrist[-main[to]+5][fr];
			else position_key ^= zobrist[6][fr];
			if(array_final[7] != 0) position_key ^= zobrist[array_final[7]-1][to];
		}
	}
	if(abs(main[to]) == 6){
		if(move==301 and main[1]==6){  /**Enroque corto, blancas**/
			if(actualiza_key){
				position_key ^= zobrist[3][0];	//Torre blanca en h1
				position_key ^= zobrist[3][2];  //Torre blanca en f1
			}
			main[0] = main[2];
			main[2] = 0;
		}    
		if(move==305 and main[5]==6){ /**Enroque largo, blancas**/	
			if(actualiza_key){
				position_key ^= zobrist[3][7];	//Torre blanca en a1
				position_key ^= zobrist[3][4];  //Torre blanca en d1
			}
			main[7] = main[4];
			main[4] = 0;
		}
		if(move==5957 and main[57]==-6){ /**Enroque corto, negras**/
			if(actualiza_key){
				position_key ^= zobrist[9][56];  //Torre negra en h8
				position_key ^= zobrist[9][58];  //Torre negra en f8
			}
			main[56] = main[58];
			main[58] = 0;
		}
		if(move==5961 and main[61]==-6){ /**Enroque largo, negras**/
			if(actualiza_key){
				position_key ^= zobrist[9][63];  //Torre negra en a8
				position_key ^= zobrist[9][60];  //Torre negra en d8
			}
			main[63] = main[60];
			main[60] = 0;
		}
	}
	if(promote){
		main[to] = array_final[7];
		if(to/8 == 0) main[fr] = -1;
		else main[fr] = 1;
	}
	/**Si captura al paso**/	
	else if(abs(main[to])==1 and to == main[69]){
		if(main[to] == 1){
			if(actualiza_key) position_key ^= zobrist[6][to-8];
			main[to-8] = -1; /**La casilla de la fila anterior de la llegada**/
		}
		else{
			if(actualiza_key) position_key ^= zobrist[0][to+8];
			main[to+8] = 1;
			}
		main[fr] = main[to];
		main[to] = 0;
	}
	else{
		main[fr] = main[to];
		main[to] = array_final[7];
	}
	if(actualiza_key){
		position_key ^= not main[68];
		position_key ^= main[68];
	}
}

void inicialize_board(tablero &input){
	tablero inicial={
		4 ,  2,  3,  6,  5,  3,  2,  4,
		1 ,  1,  1,  1,  1,  1,  1,  1,
		0 ,  0,  0,  0,  0,  0,  0,  0,
		0 ,  0,  0,  0,  0,  0,  0,  0,
		0 ,  0,  0,  0,  0,  0,  0,  0,
		0 ,  0,  0,  0,  0,  0,  0,  0,
		-1, -1, -1, -1, -1, -1, -1, -1,
		-4, -2, -3, -6, -5, -3, -2, -4,
		1, 1, 1, 1, 1, -4, 0,
		};
	for(int i=0; i<71; ++i) input[i] = inicial[i];
	}

/*****************************************
 * 	Funciones booleanas para determinar  *
 *  cosas (jaques, mate, ahogados...)    * 
 *****************************************/ 

bool ataca(const bool &blancas,const int &casilla, tablero &principal){
	for(int i=0; i<64; ++i){
		if((principal[i] > 0 and blancas) or (principal[i] < 0 and not blancas)){
			vector <int> auxiliar;
			if(abs(principal[i]) != 1) auxiliar=gen_moves(principal, i, false);
			/**Si es un peon no ataca las casillas de delante, las que se puede mover, sino las diagonales**/
			else{
				int fila=i/8;
				int col=i%8;
				if(fila+principal[i]<=7 and fila+principal[i]>=0 and col+1<8) auxiliar.push_back(i+principal[i]*8+1);
				if(fila+principal[i]<=7 and fila+principal[i]>=0 and col-1>=0) auxiliar.push_back(i+principal[i]*8-1);
				}
			for(int j=0; j<(int)auxiliar.size(); ++j){
				if(auxiliar[j] == casilla) return true;
				}
			}
		}
	return false;
	}

bool correct_move(tablero &board, int move){
	int fr=move/100, to=move%100;
	vector <int> moves = gen_moves(board, fr, true);
	for(int i=0; i<(int)moves.size(); ++i){
		if(moves[i]==to) return true;
		}
	return false;
	}

bool no_moves(tablero &board,const bool &blancas){
	if(blancas){
		for(int i=0; i<64; ++i){
			if(board[i] > 0){
				vector <int> aux = gen_moves(board, i, true);
				if(aux.size()>0) return false;
			}
		}
	}
	else{
		for(int i = 63; i >=0; --i){
			if(board[i] < 0){
				vector <int> aux = gen_moves(board, i, true);
				if(aux.size() > 0) return false;
				}
			}
		}
	return true;
	}

bool ahogado(tablero &board, bool blancas){
	if(no_moves(board, blancas)){
		if(jaque(board, blancas)) return false;
		else return true;
		}
	return false;
	}

int rbla = 3, rneg = 59;

bool jaque(tablero &board, bool blancas){
	if(blancas){
        int rey = 0;
		if(board[rbla] == 6) rey = rbla;
		else while(not (board[rey] == 6)) ++rey;
		rbla = rey;
		if(ataca(false, rey, board)) return true;
		else return false;
	}
	else{
        int rey = 63;
		if(board[rneg] == -6) rey = rneg;
		else while(not (board[rey] == -6)) --rey;
		rneg = rey;
		if(ataca(true, rey, board)) return true;
		else return false;
	}
}

/*****************************************
 * 	  Generar movimientos i generar      *
 *      movimientos movimientos para     *
 *   la qsearch (capturas i jaques)      * 
 *****************************************/ 

vector<int> gen_moves(tablero &estado, const int &from, const bool &siguente){
	int col=from%8, fil=from/8;
	vector<int> resultado;
	/**Alfil, torre y dama se generan de manera muy similar**/
	if(abs(estado[from])==3 or abs(estado[from])==4 or abs(estado[from])==5){
			vector<int> recorrei (8, 0);
			vector<int> recorrej (8, 0);
			if(abs(estado[from])==3 or abs(estado[from])==5){
				recorrei[0] = 1; recorrei[1] = 1; recorrei[2] = -1; recorrei[3] = -1;
				recorrej[0] = 1; recorrej[1] = -1; recorrej[2] = 1; recorrej[3] = -1;
				}
			if(abs(estado[from])==4 or abs(estado[from])==5){
				recorrei[4] = 0; recorrei[5] = 0; recorrei[6] = -1; recorrei[7] = 1;
				recorrej[4] = 1; recorrej[5] =-1; recorrej[6] = 0; recorrej[7] = 0;
				}
			for(int i=0; i<8; ++i){
				if(recorrei[i] == 0 and recorrej[i] == 0) continue;
				bool stop = false;
				for(int j=1; !stop; ++j){
					int ri= j*recorrei[i];
					int rj= j*recorrej[i];
					int res=(ri+rj*8)+from;
					if((rj+fil)>7 or (rj+fil)<0
					or (ri+col)>7 or (ri+col)<0) stop=true;
					
					/**Si se puede capturar, se captura
					 * tenemos que poner stop a true para que pare**/
					if(not stop){
						if((estado[from]<0 and estado[res]>0) or (estado[from]>0 and estado[res]<0)){
							resultado.push_back(res);
							stop=true;
						}
					/**Si la casilla es tuya, para**/
						if((estado[from]<0 and estado[res]<0) or (estado[from]>0 and estado[res]>0)) stop=true;
						if(estado[res]==0 and not stop){
							resultado.push_back(res);
						}
					}
					}
				}
			}
			/**Caballo**/
			else if(abs(estado[from])==2){
			/**Muchos if's para determinar si esta dentro o no
			 * Cuando se suma se mira si es mas pequeNo que 8, y cuando se resta se mira que sea mas grande que -1
			 * Tambien se debe mirar que la casilla esta vacia o no sea tuya.
			 * **/
				int llegada; /**llegada tomara valores probables donde puede ir**/ 
				bool blancas = estado[from] > 0;
				if(col+2<8 and fil+1<8){
					llegada = from + 10; /**2+(1*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
					}
				if(col+2<8 and fil-1>=0){
					llegada=from-6;/**2-(1*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col+1<8 and fil-2>=0){
					llegada= from-15;/**1-(2*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col+1<8 and fil+2<8){
					llegada=from+17;/**1+(2*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col-1>=0 and fil-2>=0){
					llegada=from-17;/**-1-(2*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col-1>=0 and fil+2<8 ){
					llegada=from+15;/**-1+(2*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col-2>=0 and fil+1<8){
					llegada=from+6; /**-2+(1*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
				if(col-2>=0 and fil-1>=0){
					llegada=from-10; /**-2-(1*8)**/
					if((blancas and estado[llegada]<=0) or (not blancas and estado[llegada]>=0)) resultado.push_back(llegada);
				}
			}
			/**Peon, se podria haber hecho para blancas y negra juntas, pero es mas leible asi**/
			else if(estado[from]==1){
				/**Si no corona**/
				if(fil<6){
					if(estado[from+8]==0){
						 resultado.push_back(from+8);
						 if(estado[from+16]==0 and fil==1) resultado.push_back(from+16);
					}
					bool captura_al_paso = (fil==4 and from+9==estado[69]);
					if(col+1 < 8 and (estado[from+9] < 0 or captura_al_paso)) resultado.push_back(from+9);
					captura_al_paso= (fil==4 and from+7==estado[69]);
					if(col-1>=0 and (estado[from+7]<0 or captura_al_paso)) resultado.push_back(from+7);
				}
				/**Si corona**/
				else{
					if(estado[from+8]==0){
						resultado.push_back(from+8+20000);   /**Caballo**/
						resultado.push_back(from+8+30000);   /**Alfil**/
						resultado.push_back(from+8+40000);   /**Torre**/
						resultado.push_back(from+8+50000);   /**Dama**/
						}
					if(estado[from+9]<0 and col+1<8){
						resultado.push_back(from+9+20000);   /**Caballo**/
						resultado.push_back(from+9+30000);   /**Alfil**/
						resultado.push_back(from+9+40000);   /**Torre**/
						resultado.push_back(from+9+50000);   /**Dama**/
						}
					if(estado[from+7]<0 and col-1>=0){
						resultado.push_back(from+7+20000);   /**Caballo**/
						resultado.push_back(from+7+30000);   /**Alfil**/
						resultado.push_back(from+7+40000);   /**Torre**/
						resultado.push_back(from+7+50000);   /**Dama**/
						}
					}
			}
			else if(estado[from]==-1){
				/**Si no corona**/
				if(fil>1){
					if(estado[from-8]==0){
						 resultado.push_back(from-8);
						 if(estado[from-16]==0 and fil==6) resultado.push_back(from-16);
					}
					bool captura_al_paso= (fil==3 and from-7 == estado[69]);
					if(col+1<8 and (estado[from-7]>0 or captura_al_paso)) resultado.push_back(from-7);
					captura_al_paso= (fil==3 and from-9==estado[69]);
					if(col-1>=0 and (estado[from-9]>0 or captura_al_paso)) resultado.push_back(from-9);
				}
				/**Si corona**/
				else{
					if(estado[from-8]==0){
						resultado.push_back(from-8+20000);   /**Caballo**/
						resultado.push_back(from-8+30000);   /**Alfil**/
						resultado.push_back(from-8+40000);   /**Torre**/
						resultado.push_back(from-8+50000);   /**Dama**/
						}
					if(estado[from-7] > 0 and col+1<8){
						resultado.push_back(from-7+20000);   /**Caballo**/
						resultado.push_back(from-7+30000);   /**Alfil**/
						resultado.push_back(from-7+40000);   /**Torre**/
						resultado.push_back(from-7+50000);   /**Dama**/
						}
					if(estado[from-9] > 0 and col-1>=0){
						resultado.push_back(from-9+20000);   /**Caballo**/
						resultado.push_back(from-9+30000);   /**Alfil**/
						resultado.push_back(from-9+40000);   /**Torre**/
						resultado.push_back(from-9+50000);   /**Dama**/
						}
					}
			}
			else if(abs(estado[from])==6){
				for(int i=-1; i<2; ++i){
					for(int j=-1; j<2; ++j){
						if((i+col<8 and i+col>=0) and (j+fil<8 and j+fil>=0)
						and ((estado[from+i+j*8]<=0 and estado[from]>0) or 
						     (estado[from+i+j*8]>=0 and estado[from]<0))) resultado.push_back(from+i+j*8);
						}
					}
				}
	/**Si la posicion inmediatamente despues de hacer el movimiento, una pieza enemiga ataca al rey
	 * eliminamos ese movimiento, ya que seria ilegal
	 * las jugadas de enroque tambien se generan aqui, ya que sino crearia un bucle infinito
	 * juto la funcion "ataca"**/
	if(siguente){
		/**Enroques**/
		/**Una norma dice que si la casilla donde ira la torre esta amenazada
		* no se puede enrocoar**/
		if(abs(estado[from]) == 6){
			if(estado[64]==1 and estado[1]==0 and estado[2]==0 and estado[from] == 6){
				if(not ataca(false, 2, estado) and not jaque(estado, true)) resultado.push_back(01);
			}
			if(estado[65]==1 and estado[4]==0 and estado[5]==0 and estado[6]==0 and estado[from] == 6){
				if(not ataca(false, 4, estado) and not jaque (estado, true))resultado.push_back(05);
			}
			if(estado[66]==1 and estado[57]==0 and estado[58]==0 and estado[from] == -6){
				if(not ataca(true, 58, estado) and not jaque(estado, false))resultado.push_back(57);
			}
			if(estado[67]==1 and estado[60]==0 and estado[61]==0 and estado[62]==0 and estado[from] == -6){
				 if(not ataca(true, 60, estado) and not jaque(estado, false))resultado.push_back(61);
			}
		}
		bool blancas = estado[from] > 0;
		vector <int> array_final(8);
		for(int i = 0; i < 7; ++i) array_final[i] = estado[64+i];
		for(int i = 0; i < (int)resultado.size(); ++i){
			array_final[7] = estado[(resultado[i]%100)];
			makemove(from*100+resultado[i],estado, false);
			if(jaque(estado, blancas)){
				unmakemove(from*100+resultado[i], estado, array_final, false);
				resultado.erase(resultado.begin()+i);  //Se tiene que pasar un iterador
				i--;
				}
			else unmakemove(from*100+resultado[i], estado, array_final, false);
			}
		}
	return resultado;
	}

vector<int> gen_caps(tablero &estado, const int &from, const bool &siguente){
	vector <int> res = gen_moves(estado, from, siguente);
	vector <int> array_final (8);
	for(int i = 0; i < 7; ++i) array_final[i] = estado[64+i];			
	for(int i = 0; i < (int) res.size(); ++i){
		/**Estos if's son si(no corona y la casilla a la que va esta vacia y no da jaque) Eliminalo**/
		if(res[i] > 10000) continue;
		if(estado[res[i]] == 0){
			bool blancas = estado[from] > 0;
			array_final[7] = estado[(res[i])];  //Modulo 100: para la coronacion
			makemove(from*100+res[i], estado, false);
			bool check = jaque(estado, not blancas);
			unmakemove(from*100+res[i], estado, array_final, false);
			if(not check){
				res.erase(res.begin()+i);
				--i;
				}
			}
		}
	return res;
}

vector<int> gen_all_moves(tablero &estado, bool blancas, bool capturas){
	vector <int> all_moves;
	for(int i = 0; i < 64; ++i){
		if((estado[i] > 0 and blancas) or (estado[i] < 0 and not blancas)){
			vector <int> aux; 
			if(not capturas) aux = gen_moves(estado, i, true);
			else aux = gen_caps(estado, i, true);
			for(int j = 0; j < (int) aux.size(); ++j) all_moves.push_back(i*100+aux[j]);
			}
		}
	return all_moves;
	}

/*****************************************
 *                                       *
 *        FUNCIONES DEL HASH             *
 *                                       * 
 *****************************************/
 
unsigned long long int seed = 0; 
 
unsigned long long int lngrand(){
	 return seed = ((seed * 264142151) + 4578531) % 18446744073709551615ULL;
	 }
	 
unsigned long long int calcula_key(const tablero &b){
	unsigned long long int key = 1;
	for(int i = 0; i < 64; ++i){
		if(b[i] == 0) continue;
		if(b[i] > 0) key ^= zobrist[b[i]-1][i];
		else key ^= zobrist[-b[i]+5][i];
		}
	return key ^ b[68];
	}
 
void inicia_hash(){
	 srand(clock());
	 for(int i = 0; i < 12; ++i){
		 for(int j = 0; j < 64; ++j){
			 zobrist[i][j] = lngrand();
			 }
		 }
	 } 

/*****************************************
 *      Search y Quiescence search       *
 *         con alpha y beta al           *
 *          estilo Negamax               * 
 *****************************************/ 

bool ordenar(const int &a, const int &b){
	if(posicion_anterior){
		if(ultima_mejorjugada[position_key] == a) return true;
		if(ultima_mejorjugada[position_key] == b) return false;
	}
	if(a >10000 or b > 10000) return a > b;
	if(a == b) return 0;
	int fra = a/100, toa = a%100, frb = b / 100, tob =  b%100;
	int difa = abs(principal[toa]) - abs(principal[fra]), difb = abs(principal[tob]) - abs(principal[frb]);
	if(difa > difb) return true;
	if(difb > difa) return false;
	if(abs(principal[toa]) > abs(principal[tob])) return true;
	if(abs(principal[tob]) > abs(principal[toa])) return false;
	if(abs(toa-27) > abs(tob-27)) return false;
	if(abs(tob-27) > abs(toa-27)) return true;
	return a > b;
	}

int search(tablero &board, int alpha, int beta, int profundidad){
	/* if(position_key != calcula_key(board)) cout << "ERROR" << endl; */
	if(board[70] >= 100) return 0;
	if(no_moves(board, board[68])){
		if(ahogado(board, board[68])) return 0;
		else return -500000-profundidad;
		}
	if(profundidad == 0){
		if(valoraciones.find(position_key) != valoraciones.end()) return valoraciones[position_key];
		int x = qsearch(board, alpha, beta, qprof);
		valoraciones[position_key] = x;
		return x;
	}
	++nodos;
	vector <int> moves = gen_all_moves(board, board[68], false);
	if(ultima_mejorjugada.find(position_key) != ultima_mejorjugada.end()) posicion_anterior = true;
	else posicion_anterior = false;
	sort(moves.begin(), moves.end(), ordenar);
	vector <int> array_final (8);
	for(int i = 0; i < 7; ++i) array_final[i] = board[64+i];
	for(int i = 0; i < (int) moves.size(); ++i){
		array_final[7] = board[moves[i]%100];
		makemove(moves[i], board, true);
		int score = -search(board, -beta, -alpha, profundidad-1);
		unmakemove(moves[i], board, array_final, true);
				
		if(alpha < score){
			if(score >= beta) return beta;
			mejormove[profundidad] = moves[i];
			alpha = score;
			ultima_mejorjugada[position_key] = moves[i];
			}
		}
	return alpha;
	}

int qsearch(tablero &board, int alpha, int beta, int profundidad){
	++nodos;
	if(no_moves(board, board[68])){
		if(ahogado(board, board[68])) return 0;
		else return -500000-profundidad;
		}
	if(profundidad == 0) return evaluate(board, board[68]);
	int x = evaluate(board, board[68]);
	if(x >= beta) return beta; // Demasiado optimista;
	vector <int> moves = gen_all_moves(board, board[68], true);
	if(moves.size() == 0) return evaluate(board, board[68]);
	if(ultima_mejorjugada.find(position_key) != ultima_mejorjugada.end()) posicion_anterior = true;
	else posicion_anterior = false;
	sort(moves.begin(), moves.end(), ordenar);
	vector <int> array_final (8);
	for(int i = 0; i < 7; ++i) array_final[i] = board[64+i];
	for(int i = 0; i < (int) moves.size(); ++i){
		array_final[7] = board[moves[i]%100];
		makemove(moves[i], board, true);
		int score = -qsearch(board, -beta, -alpha, profundidad-1);
		unmakemove(moves[i], board, array_final, true);
		
		if(score > alpha){
            alpha = score;
            ultima_mejorjugada[position_key] = moves[i];
			if(score >= beta) return beta; //Demasiado bueno para ser verdad
			}
		}
	return alpha;
	}

/*****************************************
 *                                       *
 *        FUNCION DE EVALUACION          *
 *                                       * 
 *****************************************/ 

/**Las arrays usadas en la evaluacion, las conocidas "pcsq" (piece/square table)**/
int material[7] ={0, 100, 300, 320, 500, 900, 0};

int centro[64]={
	0,   1,   3,   5,   5,   3,   1,  0,
	1,   2,   5,   8,   8,   5,   2,  1,
	3,   5,   8,  10,  10,   8,   5,  3,
	5,   8,  10,  15,  15,  10,   8,  5,
	5,   8,  10,  15,  15,  10,   8,  5,
	3,   5,   8,  10,  10,   8,   5,  3,
	1,   2,   5,   8,   8,   5,   2,  1,
	0,   1,   3,   5,   5,   3,   1,  0,
	};
/*
int alfiles[64]={
	 -5, -10, -10, -10, -10, -10, -10,  -5,
	-10,   2,  -5,   2,   2,  -5,   0, -10,
	-10,   0,  10,   0,   0,  10,   0, -10,
	-10,   2,   5,  20,  20,   5,   2, -10,
	-10,   2,   5,  20,  20,   5,   2, -10,
	-10,   0,  10,   7,   7,  10,   0, -10,
    -10,   0,  -5,   2,   2,  -5,   0, -10,
     -5, -10, -10, -10, -10, -10, -10,  -5,
	};
*/	
int caballos[64]={
	-20, -10, -10, -10, -10, -10, -10, -20,
	-10,  -5,  -5,   0,   0,  -5,  -5, -10,
	-5,    0,  10,   5,   5,  10,   0,  -5,
	-5,    5,   7,  15,  15,   7,   5,  -5,
	-5,    5,   7,  15,  15,   7,   5,  -5,
	-5,    7,   7,   7,   7,   7,   7,  -5,
   -10,   -5,  -5,   0,   0,  -5,  -5, -10,
   -20,  -10, -10, -10, -10, -10, -10, -20,
	};
	
int avance_de_peones[64]={
	0,  0,  0,  0,  0,  0,  0,  0,
	0,  0, -2, -5, -5, -2,  0,  0,
	2,  2,  2,  2,  2,  2,  2,  2,
	5,  5,  5,  5,  5,  5,  5,  5,
   10, 10, 10, 10, 10, 10, 10, 10,
   25, 25, 25, 25, 25, 25, 25, 25,
   50, 50, 50, 50, 50, 50, 50, 50,
	0,  0,  0,  0,  0,  0,  0,  0,
	};

/**Es para castigar al rey mal situado**/	
int rey[64]={
	0,   20,  -5, -10,  -5,  10,  20, -10,
   -10, -10, -10, -10, -10, -10, -10, -10,
   -30, -30, -30, -30, -30, -30, -30, -30,
   -35, -35, -35, -35, -35, -35, -35, -35,
   -50, -50, -50, -50, -50, -50, -50, -50,
   -65, -65, -65, -65, -65, -65, -65, -65,
   -80, -80, -80, -80, -80, -80, -80, -80,
   -80, -80, -80, -80, -80, -80, -80, -80,
	};
	
int rey_final[64]={
     0,  10,  10,  10,  10,  10,  10,   0,
     5,  15,  15,  15,  15,  15,  15,   5,
    10,  20,  20,  25,  25,  20,  20,  10,
    15,  30,  35,  40,  40,  35,  30,  15,
    15,  30,  35,  40,  40,  35,  30,  15,
    10,  20,  20,  25,  25,  20,  20,  10,
     5,  15,  15,  15,  15,  15,  15,   5,
     0,  10,  10,  10,  10,  10,  10,   0,
	};

/**Un poquito de trampa: para calcular los "bonus" de las negras del cuadro "c"
 * se utiliza la flip array, de manera que es array_bonus[flip[c]]
 * **/
int flip[64]={
	63, 62, 61, 60, 59, 58, 57, 56,
	55, 54, 53, 52, 51, 50, 49, 48,
	47, 46, 45, 44, 43, 42, 41, 40,
	39,	38, 37, 36, 35, 34, 33, 32,
	31,	30, 29, 28, 27, 26, 25, 24,
	23,	22, 21, 20, 19, 18, 17, 16,
	15, 14, 13, 12, 11, 10,  9,  8,
	7,   6,  5,  4,  3,  2,  1,  0,
	};

int estructura_de_peones(vector <int> &columnas_blancas, vector <int> &columnas_negras){
	sort(columnas_blancas.begin(), columnas_blancas.end());
	sort(columnas_negras.begin(), columnas_negras.end());
	int score = 0;
	vector <int>::iterator itcbb = columnas_blancas.begin(), itcbe = columnas_blancas.end(), itcnb = columnas_negras.begin(), itcne = columnas_negras.end();
	for(int i = 0; i < (int)columnas_blancas.size(); ++i){
		//Peon pasado: aquel que no tiene a ningun peon enemigo tapandole el camino
		if(find(itcnb, itcne, columnas_blancas[i]) == itcne){
			if(find(itcnb, itcne, columnas_blancas[i]+1) == itcne){
				if(find(itcnb, itcne, columnas_blancas[i]-1) == itcne){
					score += 20;
				}
			}
		}
		//Peon aislado: aquel que jamas podra ser protegido por otro peon
		if(find(itcbb, itcbe, columnas_blancas[i]+1) == itcbe or columnas_blancas[i] >= 7){
			if((find(itcbb, itcbe, columnas_blancas[i]-1) == itcbe) or columnas_blancas[i] <= 0){
				score -= 10;
			}
		}
		//Peones doblados: los que estan en fila india
		if(i == 0) continue;
		if(columnas_blancas[i] == columnas_blancas[i-1]) score -= 14;
		}
	for(int i = 0; i < (int)columnas_negras.size(); ++i){
		//Peon pasado
		if(find(itcbb, itcbe, columnas_negras[i]) == itcbe){
			if(find(itcbb, itcbe, columnas_negras[i]+1) == itcbe){
				if(find(itcbb, itcbe, columnas_negras[i]-1) == itcbe){
					score -= 20;
				}
			}
		}
		//Peon aislado
		if(find(itcnb, itcne, columnas_negras[i]+1) == itcne){
			if(find(itcnb, itcne, columnas_negras[i]-1) == itcne){
				score += 10;
			}
		}
		//Peones doblados
		if(i == 0) continue;
		if(columnas_negras[i] == columnas_negras[i-1]) score += 14;
		}
	return score;
	}

int evaluate(tablero &board, bool blancas){
	if(evaluaciones.find(position_key) != evaluaciones.end()) return evaluaciones[position_key];
	int score = 0;
	/**Material**/
	int piezas = 0; //contamos las piezas de un bando, para ver si debemos usar la array "rey" o "rey_final"
	vector <int> columnas_blancas, columnas_negras;
	for(int i = 0; i < 64; ++i){
		if(board[i] == 0) continue;
		else if(board[i] > 0){
			score += material[(int)board[i]];
			score += centro[i];
			if(board[i] == 1) columnas_blancas.push_back(i%8);
		}
		else{
			score -= material[abs(board[i])];
			score -= centro[i];
			if(board[i] == -1) columnas_negras.push_back(i%8);
		}
		piezas += material[abs(board[i])];
	}
	score += estructura_de_peones(columnas_blancas, columnas_negras); //estructura de peones de blancas - estructura de peones de negras
	/**pcsq y mobilidad de las piezas**/
	vector <int> aux;
	for(int i = 0; i < 64; ++i){
		if(board[i] == 0) continue;
		switch(board[i]){
			case 1:
				score += avance_de_peones[i];
				break;
			case -1:
				score -= avance_de_peones[flip[i]];
				break;
			case 2:
				score += caballos[i];
				break;
			case -2:
				score -= caballos[flip[i]];
				break;
			case 3:
				aux = gen_moves(board, i, false);
				score += aux.size()*2;
				break;
			case -3: 
				aux = gen_moves(board, i, false);
				score -= aux.size()*2;
				break;
			case 4:
				aux = gen_moves(board, i, false);
				score += -6+aux.size()*2;
				break;
			case -4:
				aux = gen_moves(board, i, false);
				score -= +6+aux.size()*2;
				break;
			case 5:
				aux = gen_moves(board, i, false);
				score += aux.size()*1;
				score += centro[i]-5;
				break;
			case -5:
				aux = gen_moves(board, i, false);
				score -= aux.size()*1;
				score -= centro[i]-5;
				break;
			case 6:                
				if(piezas <= 2500) score += rey_final[i];
				else score += rey_final[i];
				break;
			case -6:
				if(piezas <= 2500) score -= rey_final[flip[i]];
				else score -= rey_final[flip[i]];
				break;
				}
			}
	if(not blancas){
		evaluaciones[position_key] = -score;
		return -score;
		}
	else{
		evaluaciones[position_key] = score;
		return score;
		}
	}

/*****************************************
 *                                       *
 *                MAIN                   *
 *                                       * 
 *****************************************/ 
int main(){
	inicialize_board(principal);
	inicia_hash();
	position_key = calcula_key(principal);
	vector <int> partida;
	string menu;
	print_board(principal);
	cout << "Escribe "<< '"' << "help" << '"' <<" para ver los comandos Utiles" << endl;
	while(getline(cin, menu)){
		istringstream iss (menu,istringstream::in);
		string accion;
		iss >> accion;
		if(accion == "help"){
			cout << "Escribe los siguientes comandos para interactuar" << endl;
			cout << "pb \t\t->\t Imprime el tablero (siglas de print board)" << endl;
			cout << "new \t\t->\t Inicializa el tablero para jugar" << endl;
			cout << "white \t\t->\t Haz que la mAquina juegue con blancas, si ya jugaba deja de jugar" << endl;
			cout << "black \t\t->\t Haz que la mAquina juegue con negras, si ya jugaba deja de jugar" << endl;
			cout << "undo \t\t->\t Deshaz el Ultimo movimiento" << endl;
			cout << "draw \t\t->\t Haz que la partida acabe en tablas" << endl;
			cout << "resign \t\t->\t Uno de los dos ha perdido" << endl;
			cout << "mklog \t\t->\t El programa hara el log de la partida actual" << endl;
			cout << "depth n \t->\t El programa pensarA a una profundidad de n (n < 6)" << endl;
			cout << "qdepth n \t->\t La bUsqueda quiesciente (segunda bUsqueda) pensarA a una profundidad de n (n <= 4)" << endl;
			cout << "analize n \t->\t El programa analizarA la posicion actual a n plies (niveles de profundidad)" << endl;
			cout << "off \t\t->\t Acaba con el programa" << endl << endl;
			cout << "Para avanzar una jugada (tanto si es tuya o del rival) entra una linea en blanco" << endl;
			}
		else if(accion == "pb") print_board(principal);
		else if(accion == "new"){
			nuevo:
			partida.erase(partida.begin(), partida.end());
			inicialize_board(principal);
			position_key = calcula_key(principal);
			}
		else if(accion == "white"){
			maquina_es_blancas = not maquina_es_blancas;
			if(maquina_es_blancas) cout << "Ahora VIC juega con blancas" << endl;
			else cout << "VIC deja de jugar con blancas" << endl;
		}
		else if(accion == "black"){
			maquina_es_negras = not maquina_es_negras;
			if(maquina_es_negras) cout << "Ahora VIC juega con negras" << endl;
			else cout << "VIC deja de jugar con negras" << endl;
		}
		else if(accion == "undo"){
			/**Forma de hacer ineficiente, pero sencilla**/
			if(partida.size() > 0){
				--posiciones[position_key];
				inicialize_board(principal);
				position_key = calcula_key(principal);
				for(int i = 0; i < (int)partida.size()-1; ++i) makemove(partida[i] , principal, true);
				partida.erase(partida.end()-1);
			}
			else cout << "No hay mas movimientos" << endl;
			}
		else if(accion == "draw" or accion == "resign"){
			if(accion == "draw") cout << "La partida ha finalizado en tablas\n Desea hacer el log de la partida? (s/n)" << endl;
			else cout << "Usted o VIC ha abandonado" << endl << "Desea hacer el log de la partida? (s/n)" << endl;
			string a;
			cin >> a;
			if(a[0] == 'S' or a[0] == 's') hacer_log(partida);
			cout << "Escriba " << '"' << "new" << '"' << " para volver a empezar" << endl;
			}
		else if(accion == "mklog"){
			/**En realidad esta pregunta es para tener un retorno de carro en la entrada**/
			cout << "Desea hacer el log de la partida?" << endl;
			string a;
			cin >> a;
			if(a[0] == 's' or a[0] == 'S') hacer_log(partida);
			else getline(cin, a); //Salto
		}
		else if(accion == "depth"){
			int i;
			iss >> i;
			if(i > 6 or i <= 0) cout << "Demasiado!! Elija otra profundidad" << endl;
			else prof = i;
		}
		else if(accion == "qdepth"){
             int i;
             iss >> i;
             if(i < 0 or i > 4) cout << "Demasiado!!! Elija otra profundidad" << endl;
             else qprof = i;
             }
		else if(accion == "analize"){
			int i;
			iss >> i;
			if(i > 6 or i <= 0) cout << "Demasiada profundidad, repita el comando con menos" << endl;
			else{
				cout << "Profundidad \t Nodos \t Evaluacion \t Tiempo" << endl;
				cout << "---------------------------------------------------------------" << endl;
				for (int i = 1; i <= prof; ++i){
					position_key = calcula_key(principal);
					nodos = 0;						clock_t inicial = clock();
					int eva = search(principal, -999999999, 999999999, i);
					clock_t final = clock();
					cout << i << "\t\t" << nodos << "\t" << eva << "\t\t" << (double)(final-inicial)/CLOCKS_PER_SEC << endl;
					}
				cout << "HarA la jugada: " << 	enterohumano(mejormove[prof], principal, false) << endl;
				}
			}
		else if(accion == "off"){ cout << "ADIOS!!!" << endl; break;}
		else if(accion == ""){
			if(not no_moves(principal, principal[68]) and (posiciones[position_key] <= 2) and principal[70] < 100){
				if((principal[68] and maquina_es_blancas) or (not principal[68] and maquina_es_negras)){
					cout << "Profundidad \t Nodos \t EvaluaciOn \t Tiempo \t LInea" << endl;
					cout << "---------------------------------------------------------------" << endl;
					//vector <int> evas (prof+1);
					for (int i = 1; i <= prof; ++i){
						position_key = calcula_key(principal);
						nodos = 0;
						clock_t inicial = clock();
						int eva = search(principal, -999999999, 999999999, i);
						//if(i == 1) evas[i] = search(principal, -999999999, 999999999, i);
						//else evas[i] = search(principal, evas[i-1]-300, evas[i-1]+300, i);
						clock_t final = clock();
						cout << i << "\t\t" << nodos << "\t" << eva << "\t\t" << (double)(final-inicial)/CLOCKS_PER_SEC;
						cout << "\t\t"; //<< enterohumano(mejormove[i], principal, false) << endl;
						tablero copia;
						for(int j = 0; j < 71; ++j) copia[j] = principal[j];
						for(int j = i; j > 0; --j){
							int movimiento = ultima_mejorjugada[calcula_key(copia)];
							cout << enterohumano(movimiento, copia, false) << " " ;
							makemove(movimiento, copia, false);
							}
						cout << endl;
						valoraciones.erase(valoraciones.begin(), valoraciones.end());
						evaluaciones.erase(evaluaciones.begin(), evaluaciones.end());
						/**Tenemos mate, no perdamos el tiempo**/
						if(abs(eva) >= 500000) break;
						}	 
					cout << "Mi movimiento es " << enterohumano(mejormove[prof], principal, false) << endl;
					makemove(mejormove[prof], principal, true);
					++posiciones[position_key];
					partida.push_back(mejormove[prof]);
					ultima_mejorjugada.erase(ultima_mejorjugada.begin(), ultima_mejorjugada.end());
				}
			else{
				int move = menu_a_prueva(principal);
				string ignora;
				getline(cin,  ignora); //Salto de linea
				partida.push_back(move);
				makemove(move, principal, true);
				++posiciones[position_key];
				}
			}
			else{
				string bando;
				if(principal[68]) bando = "blancas";
				else bando = "negras"; 
				if(ahogado(principal, principal[68])) cout << "Las " << bando << " estan ahogadas" << endl;
				else{
					if(jaque(principal, principal[68]))cout << "Las " << bando << " han recibido mate" << endl;
					else if(principal[70] >= 100) cout << "Regla de los 50 movimientos, tablas" << endl;
					else cout << "RepetciOn de jugadas, tablas" << endl;
				}
				cout << "Desea hacer el log de la partida (s/n)" << endl;
				string a;
				cin >> a;
				if(a[0] == 's' or a[0] == 'S') hacer_log(partida);
				else getline(cin, a);
				goto nuevo;
				}
		}
		else cout << "Comando desconocido, para mAs ayuda escriba "<< '"' << "help" << '"' << endl;
	print_board(principal);
	}
}
